@@ -9,8 +9,10 @@
 # https://stackoverflow.com/questions/22436789/xpath-ends-with-does-not-work
 # f.find_all('xpath','//*[starts-with(@id,"z_")]')  #contains(), but matches() ends-with() not supported by webdriver/xpath1.0
 # f.find_all('xpath','//*[contains(@id,"time")][contains(@title,"Date")]')
-# f.findsel('id','activity--history-range-dropdown',timeout=10).select_by_visible_text('history')
-#                                                              .select_by_index(2)  # 0-based
+# f.findsel('id','activity--history',timeout=10).select_by_visible_text('history')
+#                                               .select_by_value()
+#                                               .select_by_index(2)  # 0-based
+#                                               .options[1].text  # first option's text
 # f.save_screenshot()                                        | f.snap()
 # f.js_click(e)                                              | f.click(e)
 # f.js_rightclick(e)                                         | f.rclick(e)
@@ -38,7 +40,7 @@
 
 # System
 from typing import Optional, Union, List, Dict, Callable, Tuple
-import pickle, os, time, json, inspect, platform, tempfile
+import pickle, os, time, json, inspect, platform, tempfile, re
 
 # Pip
 from selenium.webdriver.support.ui import Select
@@ -488,25 +490,29 @@ class Firefox:
             self.driver.set_page_load_timeout(300)
             return False
 
-    def getreq(self,url:str,regexpat,timeout=300,raw=False):
+    def getreq(self,url=None,regexpat,timeout=300,raw=False):
         """
+        url: if None, will search visited urls 
         regexpat: The pat attribute will be matched within the request URL. pat a regular expression
         if timeout and/or no match, returns ''
         else returns 
             if not raw: request.url
             else request (so that can check request.headers, request.response.headers, request.response.status_code etc)
         """
-        
         # https://pypi.org/project/selenium-wire/
-        # Access requests via the `requests` attribute
-        # for request in driver.requests:
-        # for request in driver.iter_requests():
-        #     if request.response:
-        #         print(
-        #             request.url,
-        #             request.response.status_code,
-        #             request.response.headers['Content-Type']
-        #         )
+
+        if url is None:
+            # Access requests via the `requests` attribute
+            # for request in driver.requests:
+            # request.url, request.response.status_code, request.response.headers['Content-Type']
+            for request in driver.iter_requests():
+                if request.response:
+                    if re.search(regexpat,request.url):
+                        if not raw:
+                            return request.url
+                        else:
+                            return request
+            return ''
 
         # delete previous requests to get fresh results
         del self.driver.requests
@@ -555,8 +561,6 @@ class Firefox:
         timeout: Optional[int] = None,
         element: Optional = None
     ) -> Union[Optional[WebElement], List[WebElement]]:
-        # if space in class name, convert to .
-        key=key.strip().replace(' ','.')
         return self.__find(
             by,
             EC.presence_of_element_located,
@@ -1038,10 +1042,10 @@ class Firefox:
             By.TAG_NAME: By.TAG_NAME,
         }
         by=bydict[by]
+        key = key.strip()
         if by == By.CLASS_NAME:
-            # replace space with ., which is required by xpath?? Jerry
+            # if space in class name, convert to . Jerry
             key = key.replace(" ",".")
-
         if element is None:
             element = self.driver
         elif by == By.XPATH and not key.startswith('.'):
